"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
/**
 * An ESLint rule to enforce TypeScript strict mode.
 */
const requireStrictMode = {
    meta: {
        type: "problem",
        docs: {
            category: "Possible Errors",
            description: "Enforces the use of TypeScript's strict mode.",
            recommended: "error",
            url: "https://github.com/danielnixon/eslint-plugin-total-functions",
        },
        messages: {
            strict: "TypeScript's strict mode is required.",
            noUncheckedIndexedAccess: "TypeScript's noUncheckedIndexedAccess mode is required.",
            strictFunctionTypes: "Do not disable the strictFunctionTypes compiler option.",
            strictBindCallApply: "Do not disable the strictBindCallApply compiler option.",
            strictNullChecks: "Do not disable the strictNullChecks compiler option.",
            strictPropertyInitialization: "Do not disable the strictPropertyInitialization compiler option.",
        },
        schema: [],
    },
    create: (context) => {
        const parserServices = experimental_utils_1.ESLintUtils.getParserServices(context);
        const options = parserServices.program.getCompilerOptions();
        return {
            Program: (node) => {
                const mustBeEnabled = ["strict", "noUncheckedIndexedAccess"];
                const mustNotBeDisabled = [
                    "strictFunctionTypes",
                    "strictBindCallApply",
                    "strictNullChecks",
                    "strictPropertyInitialization",
                ];
                // eslint-disable-next-line functional/no-expression-statement
                mustBeEnabled.forEach((option) => {
                    // eslint-disable-next-line functional/no-conditional-statement
                    if (options[option] !== true) {
                        // eslint-disable-next-line functional/no-expression-statement
                        context.report({
                            node: node,
                            messageId: option,
                        });
                    }
                });
                // eslint-disable-next-line functional/no-expression-statement
                mustNotBeDisabled.forEach((option) => {
                    // eslint-disable-next-line functional/no-conditional-statement
                    if (options[option] === false) {
                        // eslint-disable-next-line functional/no-expression-statement
                        context.report({
                            node: node,
                            messageId: option,
                        });
                    }
                });
            },
        };
    },
};
exports.default = requireStrictMode;
